' Functions defined for the program Arduino Reciever
' Functions defined before USE do not require declarations
' Place this module at the TOP of the main source module
'
' ---- libraries  --------------------------------------------------------------
#include Once "string.bi"	                    ' needed for format function
#include once "crt.bi"                          ' for file flush (see notes)
#include once "file.bi"                         ' for file flush (see notes)

' include global varriables
#include once "arduino_receiver_Globals.Bas"    ' Global Variables
'
'===============================================================================
Function GetIniFileName (HelpStr as String, Default as String) as Byte
    ' This function is used to get the ini file name as well as 
    ' check for user requested help.
    ' If help request is found this function terminates the program
    ' If INI file is not found this function terminates the program
    ' Input:
    '   HelpStr = string to be printed if help is requested on command line
    ' OutPut:
    '   Returns INI file name. Aborts if file not found.
    ' --------------------------------------------------------------------------
    Dim Buffer As String            ' used to read command line arguments
    Dim P as Byte                   ' working numeric variable 
    Dim TryString as String         ' Various INI names tried
    '---------------------------------------------------------------------------
    TryString="These filneames were tried:" & EOL
    ' check for command line help request
    Buffer=Trim(Ucase(Command(-1))) ' returns ENTIRE command line
    If Buffer<>"" then
        ' there are number of possible conventions
        If  (Buffer = "?") or _
            (Buffer = "HELP") or _
            (Instr(Buffer, "/?")>0) or _
            (Instr(Buffer, "/H")>0) or _
            (Instr(Buffer, "-?")>0) or _
            (Instr(Buffer, "-H")>0) or _
            (Instr(Buffer, "--?")>0) or _
            (Instr(Buffer, "--H")>0) or _
            (Instr(Buffer, "/HELP")>0) then
            Print HelpStr 
            Print "Normal exit, ExitCode: 52"
            ExitCode=52
            Return(ExitCode)
        End If
    End If
    ' Check for command line specifying INI file -------------------------------
    Buffer=Trim(Command(1))  ' returns first parameter
    If Buffer <> "" then
        ' we must have a alternate ini file name, but is it valid ?
        TryString= TryString & Buffer & EOL
        If DIR(Buffer)<>"" then 
            IniName = Buffer
            return 0
        End If
    End If
    ' Check for INI file matching EXE name -------------------------------------
    ' get the actual command string and set the default ini filename to match
    ' This allows the executable to be renamed with not change to source 
    Buffer=Trim(Command(0))                     ' includes path if used 
    if lcase(right(Buffer,4))=".exe" then 
        P=Len(Buffer)-3
        Buffer=Left(Buffer,P) & "ini"
    Else
        Buffer=Buffer & ".ini"
    End If
    ' do we have a valid file name
    TryString= TryString & Buffer & EOL
    If DIR(Buffer)<>"" then 
        IniName = Buffer
        return 0
    End If
    ' Check for default INI file name -----------------------------------------
    TryString= TryString & Default & EOL
    If DIR(Default)<>"" then 
        IniName = Default
        Return 0
    End If
    ' If we fall through to this point then report error and abort
    Print HelpStr
    Print "*** FATAL ERROR ****: No INI file found."
    Print TryString
    Print "***ERROR*** exit, ExitCode: 53"
    ExitCode=62
    Return(ExitCode)
End Function
'
'===============================================================================
Function CleanIniStr (Buffer as String, _
                ByRef KeyStr as String, _
                ByRef ValStr as String ) As Byte
    ' This function is used to strip comments, non-printing characters and
    ' quotes from a INI file String.
    ' Input:
    '   Buffer = String to be cleaned
    '   KeyStr = Holds Prarmeter name string on return
    '   ValStr = Holds Prarmeter value string on return
    ' OutPut:
    '   Returns 0  if Buffer is NOT a valid parameter line
    '   Returns -1 if Buffer is a valid parameter line
    '---------------------------------------------------------------------------
    Dim P as Byte = 0                  ' used as index into buffer
    Dim Work as String = ""            ' working storage
    Dim C as Byte                      ' holds value for single character 
    '---------------------------------------------------------------------------
    ' remove any spaces
    Buffer=Trim(Buffer)
    ' strip off comments
    P=Instr(Buffer, ";")
    If P=1 then Return 0
    If P>1 then Buffer=Trim(Left(Buffer, P-1))
    ' check for empty string
    If Buffer="" then Return 0
    ' clean up strange characters ...
    ' mostly to eliminate any tab characters or qoutes
    For P = 1 to Len(Buffer)
        C=Asc(Mid(Buffer,P,1))
        ' only accept characters that match our criteria
        ' chr(34) = double quote, chr(39) = single quote
        If (C>31) and (C<127) and (C<>34) and (C<>39) then
            Work=Work & Chr(C)
        End If
    Next P
    ' again check for spaces after possible tabs/quotes removed
    Work=Trim(Work)
    If Work="" then Return 0
    P=Instr(Work, "=")
    If (P=0) or (P=1) or (P=len(work)) then Return 0
    KeyStr=Ucase(Trim(Left(Buffer, P-1)))
    ValStr=Trim(Mid(Buffer, P+1))
    If (KeyStr="") or (ValStr="") then  Return 0
    Return -1
End Function
'
'===============================================================================
Function ReadIniFile () As Byte
    ' This function is used to read the ini file. Should be first function used.
    ' Uses previous defined function: "GetIniFileName"
    ' Uses previous defined function: "CleanIniStr"
    ' Input: 
    '   none
    ' OutPut:
    '   Returns ExitCode on Error, Else 0 
    '---------------------------------------------------------------------------
    Dim ErrCode as Integer          ' used to hold Err code
    Dim P as Byte                   ' used as index into string
    Dim Buffer as String            ' used to hold input line
    Dim KeyStr as String            ' used in parsing ini file
    Dim ValStr as String            ' used in parsing ini file
    Dim EnvStr as String            ' used to expand enviroment string
    '---------------------------------------------------------------------------
    If GetIniFileName (IdString, "Arduino_Receiver.ini")<>0 then 
        Return ExitCode
    End If
    open IniName for input as #1
    ErrCode=Err                     ' reading error code destroyes it
    If ErrCode<>0 then
        Print "*** FATAL ERROR ****: opening: " & IniName 
        Print "Error code returned was: " & ErrCode
        Print "***ERROR*** exit, ExitCode: 54"
        ExitCode=54
        Return ExitCode
    End If    
    '---------------------------------------------------------------------------
    ' read each line of the ini file checking for parameters
    While not (eof(1))
        Line Input #1, Buffer
        ErrCode=Err                     ' reading error code destroyes it
        If ErrCode<>0 then
            Print "*** FATAL ERROR **** reading: " & IniName 
            Print "Error code returned was: " & ErrCode
            Print "***ERROR*** exit, ExitCode: 55"
            ExitCode=55
            Return ExitCode
        End If
        ' CleanIniStr return 0 for non-parameter line
        If CleanIniStr(Buffer, KeyStr, ValStr)<>0 then
            ' print KeyStr, ValStr: sleep ' for debugging
            Select Case KeyStr
            Case "PORTSTR"
                PortStr=ValStr
            Case "APNDTIME"
                ValStr=Ucase(ValStr)
                if (ValStr="TRUE")  or (ValStr="YES")  then AddDateTime=1
            Case "STDOUT"
                ValStr=Ucase(ValStr)
                if (ValStr="TRUE")  or (ValStr="YES")  then StdOutFlg=1
            Case "FILESTR"
                FileStr=ValStr
                SendToFile=1
                ' expand enviromental variable
                ' I am a bit perplexed by the offsets used 
                ' but they seem to work (trial & error until success)
                If Left(FileStr,1) = "%" then
                    'Print FileStr                  'debugging
                    P=Instr(2,Buffer, "%")
                    EnvStr=Mid(Left(FileStr,P-2),2)
                    'Print EnvStr                   'debugging
                    EnvStr=Environ(EnvStr)
                    'Print EnvStr                   'debugging
                    FileStr=EnvStr & Mid(FileStr, P)
                End If
            Case "SWITCHHRS"
                ' print KeyStr, ValStr: sleep ' for debugging
                SwitchHrs=Abs(Int(Val(ValStr)))
                If SwitchHrs>24 then SwitchHrs=24
                ' print SwitchHrs, ValStr: sleep ' for debugging
            Case "DELIMITER"
                if ValStr="TAB" then: Delimiter=chr(09)
                elseif ValStr="COMMA"  then Delimiter=", "
                elseif ValStr="COLLON"  then Delimiter=": "
                elseif Abs(Val(ValStr)) >0 then _ 
                           Delimiter=Space(Int(Val(ValStr)))
                elseif Left(ValStr,1)="0" then Delimiter=""
                elseif Lcase(ValStr)="zero" then Delimiter=""
                else DELIMITER=chr(9)
                end if
            Case "EOL"
                ValStr=Left(ValStr,1)
                if ValStr="0" then: EOL=chr(13) & chr(10)
                elseif ValStr="1" then EOL=chr(10)
                elseif ValStr="2" then EOL=chr(13)
                else EOL=chr(13) & chr(10)
                end if
            Case "EXITKEY"
                If Lcase(ValStr)="escape" then 
                    ExitKey=27
                Elseif Lcase(ValStr)="ctlx" then
                    ExitKey=24
                Else
                    ExitKey=Abs(Int(Val(ValStr)))
                    If ExitKey<>0 then
                        If ExitKey>126 then ExitKey=27
                        If ExitKey<32  then ExitKey=27
                    End If
                end if
			'----------------------------------------------------------
			' add addtional case statements for new options in INI file
			' Example:
			'Case "BULLWINKLE"
			'	BULLWINKLE code goes here
			'Case "ROCKYRACOON"
			'   ROCKYRACOON code goes here
			'case keyword MUST be UPPERCASE (in INI file it can be upper or lower)
			'for reliability include spaces in keyword 
			'----------------------------------------------------------
            Case Else
                'ignore it
            End Select ' case KeyStr
        End If ' CleanIniStr
    Wend 'not (eof(1))    
    Close #1
    ErrCode=Err                     ' reading error code destroyes it
    If ErrCode<>0 then
        Print "*** FATAL ERROR **** closing: " & IniName 
        Print "Error code returned was: " & ErrCode
        Print "***ERROR*** exit, ExitCode: 56"
        ExitCode=56
        Return ExitCode
    End If 
    Return ExitCode
End Function
'
'===============================================================================
Function BuildFileStr () as String
    ' This function is used build sequential file names when SwitchHrs>0
    ' Input: 
    '   none
    ' OutPut:
    '   Returns valid FileStr UNLESS SendToFile=0
    '---------------------------------------------------------------------------
    Static BaseFileStr0 as String       ' holds FileStr as defined in INI file
    Static BaseFileStr1 as String       ' holds first part of FileStr
    Static BaseFileStr2 as String       ' holds last part of FileStr
    Static SwitchHrsNdx as Integer=-1   ' holds current iteration of filename
    '---------------------------------------------------------------------------
    Dim P as Integer                    ' index into string
    Dim Exist as String="xxx"           ' used to check for existence of file 
    '---------------------------------------------------------------------------
    ' by definition if SendToFile=0 then FileStr=""
    If SendToFile=0 then Return ""
    ' If SwitchHrs=0 then default FileStr is to be used
    If SwitchHrs=0 then Return FileStr
    ' This is only executed on the first call
    If SwitchHrsNdx = -1 then 
        ' This is the first time into the function
        P=Instr(FileStr, "0000")
        If P=0 then
            ' user has failed to specify appropriate file string
            ' disable SwitchHrs and return
            SwitchHrs=0
            Return FileStr
        Else
            BaseFileStr0=FileStr         ' save it just in case
            BaseFileStr1=Left(FileStr,P-1)
            BaseFileStr2=Mid(FileStr,P)
            ' regardless of what was in the INI file 
            ' we are only going to use four zeros
            While Left(BaseFileStr2,1)="0"
               BaseFileStr2=Mid(BaseFileStr2,2)
            Wend
        End If
    End If
    ' This is executed on the every call
    While Exist <> ""
        SwitchHrsNdx=SwitchHrsNdx +1
        FileStr=BaseFileStr1 & _
                Right("0000" & Trim(Str(SwitchHrsNdx)),4) & _
                BaseFileStr2
        Exist=Dir(FileStr)
    Wend
    Return (FileStr)
End Function
'
'===============================================================================
Function BuildStatusStr () as Byte
    ' This function is used to build the status line message that is displayed
    ' when the user presses "?" or "/".
    ' Input: 
    '   none
    ' OutPut:
    '   Returns StdOutFlg
    '---------------------------------------------------------------------------
    
    Status = EOL & _
			 "===" & ProgramName & " Ver " & ProgramVer & " ===" & EOL & _
             "Using INI file: " & EOL & IniName & EOL & _
             "Using Port Str: " & PortStr & EOL
    If AddDateTime<>0 then Status = Status & "Appending date/time to data." & EOL
    If FileStr<>"" then
        Status = Status & "Logging data to: " & FileStr & EOL
        ' display EOL and Delimiter are a bit more complicated
        ' Fortunately there are a limited numebr of possibilities.
        If Delimiter=", " then: Status = Status & "Delimiter: Comma" & EOL
        elseIf Delimiter=": " then Status = Status & "Delimiter: Collon" & EOL
        elseIf Delimiter=chr(9) then Status = Status & "Delimiter: Tab" & EOL
        elseIf Delimiter="" then Status = Status & "Delimiter: none" & EOL
        else Status = Status & "Delimiter: " & chr(34) & Delimiter & Chr(34) & EOL
        end if
        If EOL=chr(13) then: Status = Status & "EOL: carriage return" & EOL
        elseIf EOL=chr(10) then Status = Status & "EOL: line feed" & EOL
        else Status = Status & "EOL: carriage return plus line feed" & EOL
        end if
    End If
    Status = Status & "Start: " & Dstart & " " & Tstart & EOL
    Status = Status & "Press '?' or '/' to redispaly this message" & EOL
    If ExitKey = 24 then 
        Status = Status & "Press 'Ctrl' and  'X' to exit." & EOL
    ElseIf ExitKey = 27 then
        Status = Status & "Press 'Escape' key to exit." & EOL
    Else
        Status = Status & "Press '" & Chr(ExitKey) & "' key to exit." & EOL
    End If
    '---------------------------------------------
    ' Add Aplication help here
    
    Return StdOutFlg
End Function
'
'===============================================================================
Function EvalGlobals () as Byte
    ' This function is used to read the ini file. Should be first function used.
    ' Uses previous defined function: "BuildFileStr"
    ' Uses previous defined function: "BuildStatusStr"
    ' Input: 
    '   none
    ' OutPut:
    '   Returns ExitCode on failure, Else 0
    '---------------------------------------------------------------------------
    ' Port String is required
    If PortStr="" then
        Print "*** FATAL ERROR ****: PortStr not found in " & IniName
        Print "***ERROR*** exit, ExitCode: 57"
        ExitCode=57
        Return ExitCode
    End If
    ' If SwitchHrs<>0 then we need to build our FileStr
    If SwitchHrs<> 0 then FileStr=BuildFileStr
    If BuildStatusStr()<> 0 then print Status
    Return ExitCode
End Function
'
'===============================================================================
Function OpenCommunications() as Byte
    ' This function is used to open the Serial port and the optional log file
    ' Input: 
    '   none
    ' OutPut:
    '   Returns ExitCode
    '---------------------------------------------------------------------------
    Dim ErrCode as Integer          ' used to hold Err code
    '---------------------------------------------------------------------------
	' One alternative is to use this function to open a Database
	' Data could then be sent to the Database in the ProcessData function
	' Add code to CloseCommunications to close DataBase.
	'---------------------------------------------------------------------------
	Open Com(PortStr) AS #2
    ErrCode=Err                     ' reading error code destroyes it
    If ErrCode<>0 then
        Print "***Fatal Error*** opening com port using string: " 
        Print PortStr
        Print "Error code was: " & ErrCode
        Print "***ERROR*** exit, ExitCode: 58"
        ExitCode=58
    End If
    '---------------------------------------------------------------------------
    If SendToFile<>0 then 
        Open FileStr for Append as #3
        ErrCode=Err
        If ErrCode<>0 then
            Print "***Error*** opening file: " & FileStr
            Print "Error reported was: " & ErrCode
            Close #2
            print "***ERROR*** exit, ExitCode: 59"
            ExitCode=59
        End If
    End If
    Return ExitCode
End Function
'
'===============================================================================
Function CloseCommunications() as Byte
    ' This function is used to close the Serial port and the optional log file
    ' Input: 
    '   none
    ' OutPut:
    '   always Returns 0
    '---------------------------------------------------------------------------
    Dim ErrCode as Integer          ' used to hold Err code
    Dim SaveCode as Integer         ' used to hold Err code
    '---------------------------------------------------------------------------
    SaveCode = ExitCode             ' save the reason we are exiting
    Close #2
    ErrCode=Err
    If ErrCode<>0 then
        Print "***Error*** closing file COM port" 
        Print "Error reported was: " & ErrCode
        ExitCode=62
    End If
    '------------------------------------------------------
    If SendToFile<>0 then 
        ' if open failed then we never got here ...
        ' log the reason the program is exiting
        Print #3, "Exit Code: " & SaveCode & " at " & Time & " on " & Date
        Close #3
        ErrCode=Err
        If ErrCode<>0 then
            Print "***Error*** closing file: " & FileStr
            Print "Error reported was: " & ErrCode
            ExitCode=63
        End If
    End If
    '------------------------------------------------------
    Select Case SaveCode
    Case 50
        print "Normal exit, User pressed exit key"
    Case 51 
        print "Normal exit, EOT recieved in data stream"
    Case Else 
        print "***ERROR*** exit, ExitCode: " & SaveCode
    End Select
    Return ExitCode
End Function
'
'===============================================================================
Function CheckTime() as Byte
    ' This function is used tally hours for sequential files filanems.
    ' If the total hours is greater thatn ot equal to SwitchHrs then 
    ' the current log file is close and a new one is opened.
    ' Uses previous defined function: "BuildFileStr"
    ' Uses previous defined function: "BuildStatusStr"
    ' Input: 
    '   none
    ' OutPut:
    '   returns ExitCode on Failure, else 0
    '---------------------------------------------------------------------------
    ' revision October 2013 -- 
    ' NowHour, PrevHour, HourCount made into Global Variables
    ' Eliminated first pass code -- Variables initalized Globally
    ' Faster exit when hours have not changed
    ' Fixed updating PrevHour bug
    '---------------------------------------------------------------------------
    Dim ErrCode as Integer      ' used to hold Err code
    Dim SaveCode as Integer     ' used to hold Err code
    Dim DateStr as String       ' used to reformat date/time into useable format
    Dim K as Byte = 0           ' error trigger
    '---------------------------------------------------------------------------
    ' always check for unintended calls ....
    If SwitchHrs=0 then return 0
    ' check hours
    NowHour=val(Left(time,2))
    if NowHour=PrevHour then return 0
    ' ExitCode=0 ' keep current exit code unless ther is an error
    
    ' if we get this far then we need to increment our parameters
    PrevHour=NowHour
    HourCount=HourCount+1
    ' now check for the file switch
    If HourCount> SwitchHrs then
        ' time to change our socks ...
        ' print "changeing socks ..." : sleep
        Close #3
        ErrCode=Err
        If ErrCode<>0 then
            Print "***Error*** closing file during file switch: " & FileStr
            Print "Error reported was: " & ErrCode
            ExitCode = 64
            Return 64
        End If
        Open BuildFileStr() for Append as #3
        ErrCode=Err
        If ErrCode<>0 then
            Print "***Error*** opening file during file switch: " & FileStr
            Print "Error reported was: " & ErrCode
            print "***ERROR*** ExitCode: 65"
            ExitCode = 65
            Return 65
        End If
        VoidByte=BuildStatusStr()
        HourCount=0
    End If
    Return ExitCode
End Function
'
'===============================================================================
Function ProcessData(ByRef Buffer as String ) as Byte
    ' This function is used to process a single line of filtered ASCII data 
    ' that has been collected from the serial port.
    ' All data writes take place in this function.
    ' Input: 
    '   none
    ' OutPut:
    '   Returns ExitCode=60 on error, else 0
    '---------------------------------------------------------------------------
    ' revision October 2013 -- 
    ' moved checktime function call to this function
    '---------------------------------------------------------------------------
    Dim ErrCode as Integer      ' used to hold Err code
    Dim SaveCode as Integer     ' used to hold Err code
    Dim DateStr1 as String      ' used to reformat date/time into useable format
    Dim DateStr2 as String      ' used to reformat date/time into useable format
    Dim Counter as Integer      ' used to trigger CheckTime
    Dim K as Byte = 0           ' error trigger
    '---------------------------------------------------------------------------
    '---------------------------------------------------------------------------
	' Place any addition processing code in this function.
	' Keep it SHORT. Reading the COM is 'paused' for this function.
        
    '------------ display write ------
    ' next line modified for Thermometer application
    If (AddDateTime) then
        DateStr1=mid(Date,7,4) & "-"                ' get year
        DateStr1=DateStr1 & Left(Date,2)  & "-"     ' get month
        DateStr1=DateStr1 & Mid(Date,4,2)           ' get day
        DateStr2=DateStr1 & Delimiter & Time        ' get time for file
        DateStr1=DateStr1 & "  " & Time             ' get time for display
        If StdOutFlg=0 then
            Print DateStr1 & "    " & buffer
        Else
            Print DateStr2 & Delimiter & buffer
        End If
    Else
        Print buffer
    End If
    
    '------------ file write ------
    If SendToFile<>0 then 
        ' revision October 2013 -- 
        ' The only time that we actually need to check the time 
        ' is when we are writing to a file
        CheckTime
        ' setup for file flush
        LogFileHandle=cast(FILE Ptr,Fileattr(3,fbFileAttrHandle))
        If (AddDateTime) then
            Print #3,DateStr2 & Delimiter & buffer & EOL;
            ErrCode=Err
            fflush(LogFileHandle)      ' no error to check for
        Else
            Print #3, buffer & EOL;
            ErrCode=Err
            fflush(LogFileHandle)      ' no error to check for
        End If
        If ErrCode<>0 then
            Print "***Error*** writting file: " & FileStr
            Print "Error reported was: " & ErrCode
            ExitCode = 61
            Return 61
        End If
    End If ' SendToFile<>0
    '------------ clear input buffer ------
    Buffer=""
    Return ExitCode
End Function
'
'===============================================================================
Function Communications() as Byte
    ' This function is the main process loop for collecting data from both the
    ' serial port and the keyboard. All data reads take place in this function.
    ' The Serial data is filtered for ASCII characters only. When an EOL
    ' is recieved then the data string is sent to the ProcessData function.
	' Com port & optional log file must be opened prior to calling this function.
    ' Uses previous defined function: "ProcessData"
    ' Uses previous defined function: "CheckTime"
    ' Input: 
    '   none
    ' OutPut:
    '   Returns ExitCode
    '---------------------------------------------------------------------------
    ' revision October 2013 -- 
    ' moved checktime function call from this function to ProcessData function
    '---------------------------------------------------------------------------
    Dim ErrCode as Integer      ' used to hold Err code
    Dim C As Byte = 0			' this is our incoming byte of data
    Dim K as Integer =0         ' used to read keyboard & trigger exit
    Dim Buffer As String = "" 	' this is our buffer to collect the bytes
    ' Dim Counter as Integer      ' used to trigger CheckTime
    '---------------------------------------------------------------------------
    While InKey<> ""            'empty the keyboard buffer ... just in case
    Wend

    ' loop untill there is an exit trigger
    While ExitCode=0
        ' The first line checks to see if there is anything waiting in the COM 
        ' buffer. Without it one is subject to reading a bunch of garbage.
        If EOF(2) then
            ' Call Sleep with 25ms or less to release time-slice when waiting 
            ' for user input or looping inside a thread.This will prevent the 
            ' program from unnecessarily hogging the CPU.
            Sleep 25
        Else
            ' get a single byte from the serial port
            Get #2,0,C,1
            ErrCode=Err
            If ErrCode<>0 then
                Print "***Fatal Error*** reading com port" 
                Print PortStr
                Print "Error code was: " & ErrCode
                ExitCode=60
            End If
            ' ------ filter recieved data --------------------------------------
            ' characters below ASCII 32 are 'non-printing characters
            ' characters above ASCII 126 are not defined (by ASCII)
            ' append any printable character to the string 
            ' include an exception for horizontal tab characters
            If ((C > 31) and (C < 127)) or (C = 9) Then Buffer = Buffer + Chr(C)
            ' Linux/Unix terminate strings with a line feed (ASCII 10)
            ' MACs terminate lines with a carriage return (ASCII 13)
            ' Microsoft and Arduino use carriage return/linefeed (ASCII 13,10)
            ' End of Transmission is ASCII 04
            ' If we get any of the above then print the string 
            ' but only if we have something to process.
            If ((C=13) Or (C=10) or (C=04)) And  (Len(Buffer) >0) Then 
                ExitCode = ProcessData(buffer)
            End If ' ((C=13) Or (C=10) ...
            ' check for end of tranmission code in data stream
            If C=04 then 
                ExitCode=51
            End If
        End If  ' Not(EOF(2))
'        ' revision October 2013 --        
'        ' if we are writing to sequential file names then we need to
'        ' check if a log file needs to be changed
'        If SwitchHrs>0 then
'            Counter=Counter+1
'            If Counter > 2048 then      ' abitrary number, change as needed
'                ExitCode=CheckTime
'                Counter=0
'            End If
'        End If
        ' if we do not already have an exit flag then read the key  board
        If ExitCode=0 then
            K=ASC(InKey)
            Select Case K
            Case 63,47                  ' ASCII 63 ="?" question mark
                Print Status            ' ASCII 47 ="/" question mark
            Case ExitKey
                ExitCode=50             ' set exit code, normal exit
            Case Else
                ' Add additional case statements for other Keyboard codes.
				' Use TestKeyCode.exe to identify ASC() keycodes.
				' Hint:You can use mapped keys for longer sequences:
                '   Case 65,97 then print #2, "Long command line Here".
				'   "A"=65, "a"=97
				'
                ' Or place Keyboard handling code here.
                ' Example: 
                '    if you need to send special commands 
                '    out the serial port to control the Arduino
                '    A=Ucase(chr(K))
                '    If instr("0123456789ABCDEF",A) then Print #2, A;
				'
                ' Keep it SHORT ... the COM port will not be read again
                ' until this sequence is completed. 
				' It is best handle one key stroke at a time at a time.
				' Otherwise: 
				' 1) Increase the size of the recieve buffer using the PortStr
				'    extended option RB (i.e. RB64 or RB128 or RB256 or RB1024) 
                ' 2) Launch a seperate thread for keyboard handling/processing 
                '    and replace this section with access to a shared variable
				'
            End Select
        End If
    Wend
    Return ExitCode
End Function
'
'===End of File=================================================================
